@@ -2,7 +2,7 @@ import asyncio
 import unittest
 
 from .http import *
-from .http import basic_auth_header, build_headers, read_headers
+from .http import read_headers
 
 
 class HTTPAsyncTests(unittest.TestCase):
@@ -33,7 +33,7 @@ class HTTPAsyncTests(unittest.TestCase):
         path, headers = self.loop.run_until_complete(
             read_request(self.stream))
         self.assertEqual(path, '/chat')
-        self.assertEqual(dict(headers)['Upgrade'], 'websocket')
+        self.assertEqual(headers['Upgrade'], 'websocket')
 
     def test_read_response(self):
         # Example from the protocol overview in RFC 6455
@@ -48,7 +48,7 @@ class HTTPAsyncTests(unittest.TestCase):
         status_code, headers = self.loop.run_until_complete(
             read_response(self.stream))
         self.assertEqual(status_code, 101)
-        self.assertEqual(dict(headers)['Upgrade'], 'websocket')
+        self.assertEqual(headers['Upgrade'], 'websocket')
 
     def test_request_method(self):
         self.stream.feed_data(b'OPTIONS * HTTP/1.1\r\n\r\n')
@@ -102,36 +102,127 @@ class HTTPAsyncTests(unittest.TestCase):
             self.loop.run_until_complete(read_headers(self.stream))
 
 
-class HTTPSyncTests(unittest.TestCase):
+class HeadersTests(unittest.TestCase):
 
-    def test_build_headers(self):
-        headers = build_headers([
-            ('X-Foo', 'Bar'),
-            ('X-Baz', 'Quux Quux'),
+    def setUp(self):
+        self.headers = Headers([
+            ('Connection', 'Upgrade'),
+            ('Server', USER_AGENT),
         ])
 
-        self.assertEqual(headers['X-Foo'], 'Bar')
-        self.assertEqual(headers['X-Bar'], None)
-
-        self.assertEqual(headers.get('X-Bar', ''), '')
-        self.assertEqual(headers.get('X-Baz', ''), 'Quux Quux')
-
-    def test_build_headers_multi_value(self):
-        headers = build_headers([
-            ('X-Foo', 'Bar'),
-            ('X-Foo', 'Baz'),
-        ])
-
-        # Getting a single value is non-deterministic.
-        self.assertIn(headers['X-Foo'], ['Bar', 'Baz'])
-        self.assertIn(headers.get('X-Foo'), ['Bar', 'Baz'])
-
-        # Ordering is deterministic when getting all values.
-        self.assertEqual(headers.get_all('X-Foo'), ['Bar', 'Baz'])
-
-    def test_basic_auth_header(self):
-        # Test vector from RFC 7617.
+    def test_str(self):
         self.assertEqual(
-            basic_auth_header("Aladdin", "open sesame"),
-            ('Authorization', 'Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ=='),
+            str(self.headers),
+            "Connection: Upgrade\r\nServer: {}\r\n\r\n".format(USER_AGENT),
+        )
+
+    def test_repr(self):
+        self.assertEqual(
+            repr(self.headers),
+            "Headers([('Connection', 'Upgrade'), "
+            "('Server', '{}')])".format(USER_AGENT),
+        )
+
+    def test_multiple_values_error_str(self):
+        self.assertEqual(
+            str(MultipleValuesError('Connection')),
+            "'Connection'",
+        )
+        self.assertEqual(
+            str(MultipleValuesError()),
+            "",
+        )
+
+    def test_contains(self):
+        self.assertIn('Server', self.headers)
+
+    def test_contains_case_insensitive(self):
+        self.assertIn('server', self.headers)
+
+    def test_contains_not_found(self):
+        self.assertNotIn('Date', self.headers)
+
+    def test_iter(self):
+        self.assertEqual(set(iter(self.headers)), {'connection', 'server'})
+
+    def test_len(self):
+        self.assertEqual(len(self.headers), 2)
+
+    def test_getitem(self):
+        self.assertEqual(self.headers['Server'], USER_AGENT)
+
+    def test_getitem_case_insensitive(self):
+        self.assertEqual(self.headers['server'], USER_AGENT)
+
+    def test_getitem_key_error(self):
+        with self.assertRaises(KeyError):
+            self.headers['Upgrade']
+
+    def test_getitem_multiple_values_error(self):
+        self.headers['Server'] = '2'
+        with self.assertRaises(MultipleValuesError):
+            self.headers['Server']
+
+    def test_setitem(self):
+        self.headers['Upgrade'] = 'websocket'
+        self.assertEqual(self.headers['Upgrade'], 'websocket')
+
+    def test_setitem_case_insensitive(self):
+        self.headers['upgrade'] = 'websocket'
+        self.assertEqual(self.headers['Upgrade'], 'websocket')
+
+    def test_setitem_multiple_values(self):
+        self.headers['Connection'] = 'close'
+        with self.assertRaises(MultipleValuesError):
+            self.headers['Connection']
+
+    def test_delitem(self):
+        del self.headers['Connection']
+        with self.assertRaises(KeyError):
+            self.headers['Connection']
+
+    def test_delitem_case_insensitive(self):
+        del self.headers['connection']
+        with self.assertRaises(KeyError):
+            self.headers['Connection']
+
+    def test_delitem_multiple_values(self):
+        self.headers['Connection'] = 'close'
+        del self.headers['Connection']
+        with self.assertRaises(KeyError):
+            self.headers['Connection']
+
+    def test_eq(self):
+        other_headers = self.headers.copy()
+        self.assertEqual(self.headers, other_headers)
+
+    def test_eq_not_equal(self):
+        self.assertNotEqual(self.headers, [])
+
+    def test_clear(self):
+        self.headers.clear()
+        self.assertFalse(self.headers)
+        self.assertEqual(self.headers, Headers())
+
+    def test_get_all(self):
+        self.assertEqual(self.headers.get_all('Connection'), ['Upgrade'])
+
+    def test_get_all_case_insensitive(self):
+        self.assertEqual(self.headers.get_all('connection'), ['Upgrade'])
+
+    def test_get_all_no_values(self):
+        self.assertEqual(self.headers.get_all('Upgrade'), [])
+
+    def test_get_all_multiple_values(self):
+        self.headers['Connection'] = 'close'
+        self.assertEqual(
+            self.headers.get_all('Connection'), ['Upgrade', 'close'])
+
+    def test_raw_items(self):
+        self.assertEqual(
+            list(self.headers.raw_items()),
+            [
+                ('Connection', 'Upgrade'),
+                ('Server', USER_AGENT),
+            ],
         )
